const IS_DEV = process.env.NODE_ENV === "development";
function log(action, path, message) {
  if (IS_DEV) {
    console.log(`[API - ${action}] ${message}: ${path}`);
  }
}
export class GraphqlMiddlewarePlugin {
  constructor(baseURL, headers, useCache, context) {
    this.baseURL = baseURL;
    this.headers = headers || {};
    this.context = context;
    if (useCache) {
      this.cache = new Map();
    }
  }
  getPluginHeaderValue() {
    return {
      "Nuxt-Graphql-Middleware-Route": this.context?.route?.fullPath || ""
    };
  }
  beforeRequest(fn) {
    this.beforeRequestFn = fn;
  }
  query(name, variables, headers = {}) {
    const params = new URLSearchParams({
      name,
      variables: JSON.stringify(variables || {})
    });
    const url = this.baseURL + "/query?" + params.toString();
    if (this.cache?.has(url)) {
      log("query", url, "Loading from cache");
      return Promise.resolve(this.cache.get(url));
    }
    log("query", url, "Fetching");
    let fetchOptions = {
      method: "GET",
      credentials: "include",
      headers: {
        "Content-Type": "application/json",
        ...headers,
        ...this.headers,
        ...this.getPluginHeaderValue()
      }
    };
    if (this.beforeRequestFn) {
      fetchOptions = this.beforeRequestFn(this.context, fetchOptions);
    }
    return fetch(url, fetchOptions).then((response) => {
      if (response.ok) {
        return response.json();
      }
      throw new Error("Server Error");
    }).then((data) => {
      if (this.cache && this.cache.size > 30) {
        const key = this.cache.keys().next().value;
        this.cache.delete(key);
      }
      this.cache?.set(url, data);
      return data;
    });
  }
  mutate(name, variables, headers = {}) {
    const params = new URLSearchParams({
      name
    });
    let fetchOptions = {
      method: "POST",
      credentials: "include",
      headers: {
        "Content-Type": "application/json",
        ...headers,
        ...this.headers,
        ...this.getPluginHeaderValue()
      },
      body: JSON.stringify(variables)
    };
    if (this.beforeRequestFn) {
      fetchOptions = this.beforeRequestFn(this.context, fetchOptions);
    }
    return fetch(this.baseURL + "/mutate?" + params.toString(), fetchOptions).then((response) => response.json());
  }
}
